﻿using System;
using System.Text;
using System.Threading.Tasks;
using DeukBot4.Database;
using DeukBot4.MessageHandlers.CommandHandler.RequestStructure;
using DeukBot4.MessageHandlers.Permissions;
using Discord;

namespace DeukBot4.MessageHandlers.CommandHandler
{
    public class RolePermissionCommands : CommandContainerBase
    {
        public override string Name => "Role Related";

        [Command("roles", PermissionLevel.Admin)]
        [CommandHelp("Lists all roles on the server along with their IDs", "Lists all roles on the server along with their IDs")]
        [BlockUsageInPm]
        public async Task ListServerRoles(CommandRequest request)
        {
            var channel = request.OriginalMessage.Channel;
            if (!(channel is IGuildChannel serverChannel))
                return;

            var roles = serverChannel.Guild.Roles;
            var sb = new StringBuilder();
            foreach (var role in roles)
            {
                sb.Append($"``{role.Name}`` --> {role.Id}\n");
            }
            await channel.SendMessageAsync(sb.ToString());
        }

        [Command("adminrole", PermissionLevel.ServerOwner)]
        [CommandParameters(new []{ParameterMatcher.ParameterType.Number})]
        public async Task SetAdminRole(CommandRequest request)
        {
            await SetRolePermission(request, PermissionLevel.Admin);
        }

        [Command("moderatorrole", PermissionLevel.ServerOwner)]
        [CommandParameters(new []{ParameterMatcher.ParameterType.Number})]
        public async Task SetModRole(CommandRequest request)
        {
            await SetRolePermission(request, PermissionLevel.Moderator);
        }

        [Command("helperrole", PermissionLevel.ServerOwner)]
        [CommandParameters(new []{ParameterMatcher.ParameterType.Number})]
        public async Task SetHelperRole(CommandRequest request)
        {
            await SetRolePermission(request, PermissionLevel.Helper);
        }

        [Command("blockedrole", PermissionLevel.ServerOwner)]
        [CommandParameters(new []{ParameterMatcher.ParameterType.Number})]
        public async Task SetBannedRole(CommandRequest request)
        {
            await SetRolePermission(request, PermissionLevel.Banned);
        }


        private static async Task SetRolePermission(CommandRequest request, PermissionLevel permissionLevel)
        {
            var channel = request.OriginalMessage.Channel;
            if (!(channel is IGuildChannel serverChannel))
                return;

            if (request.Parameters.Length == 0)
            {
                await request.OriginalMessage.Channel.SendMessageAsync(
                    $"You did not give a valid role ID. Use ``!roles`` to list all current server roles, along with their ids");
                return;
            }

            if (!ulong.TryParse(request.Parameters[0].AsString(), out var roleId))
            {
                await request.OriginalMessage.Channel.SendMessageAsync(
                    $"You did not give a valid role ID. Use ``!roles`` to list all current server roles, along with their ids");
                return;
            }

            var role = serverChannel.Guild.GetRole(roleId);
            if (role == null)
            {
                await request.OriginalMessage.Channel.SendMessageAsync("No role with that id exists on this server");
                return;
            }

            try
            {
                await DatabaseRolePermissions.SetRolePermission(serverChannel.GuildId, roleId, permissionLevel);
                PermissionValidator.UpdateCache(serverChannel.GuildId, roleId, permissionLevel);
            }
            catch(Exception e)
            {
                await Logger.LogError(e.Message);
            }
        }

    }
}